#pragma once
#include <audioState.h>
#include <timer.h>

#include "station.h"
#include "context.h"
#include "sequence.h"

class Station;

class State
{
private:
    Timer sequenceTimer;
    bool currentEntryHandled;

protected:
    Sequence* sequence;
    int sequencePointer;
    bool sequenceDone;

public:
    virtual ~State() = default;

    explicit State(Sequence* sequence): sequenceTimer(Timer(0, false)), currentEntryHandled(false), sequence(sequence),
                                        sequencePointer(0), sequenceDone(false)
    {
    }

    virtual State* pickedUp(Context context) { return this; }
    virtual State* putDown(Context context, Station* newStation) { return this; }
    virtual State* update(Context context) { return this; }

    virtual void activated(Context context)
    {
    }

    virtual String updateDisplay() { return ""; }

    void updateSeqence(const Context& context)
    {
        if (sequence == nullptr || sequence->getLength() == 0)
        {
            return;
        }
        if (!currentEntryHandled)
        {
            const auto currentEntry = sequence->getEntryAtIndex(sequencePointer);
            const auto addressInt = reinterpret_cast<uintptr_t>(currentEntry);
            this->sequenceTimer = Timer(currentEntry->duration, false);
            this->sequenceTimer.start();
            if (currentEntry->isAudio)
            {
                // printMp3Detail(context.dfPlayer->readType(), context.dfPlayer->read());
                context.dfPlayer->play(currentEntry->audioId);
            }
            currentEntryHandled = true;
        }
        const bool timeOut = this->sequenceTimer.update(context.delta);
        if (timeOut)
        {
            if (this->sequencePointer + 2 >= this->sequence->getLength())
            {
                sequencePointer = sequence->getRepeatIndex();
                sequenceDone = true;
                currentEntryHandled = false;
                return;
            }
            this->sequencePointer += 1;
            currentEntryHandled = false;
        }
    }
};
